{#
# This file is part of Invenio.
# Copyright (C) 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#}

{%- extends "invenio_oauth2server/settings/index.html" %}

{%- import "invenio_oauth2server/settings/helpers.html" as helpers with context %}
{%- from "invenio_oauth2server/settings/_macros.html" import render_field with context %}

{% block settings_content %}
{{helpers.panel_start(
    _('Personal access token / %(name)s', name=token.client.name),
    with_body=False,
)}}
{% if show_token %}
<div class="panel-body">
    <div class="row">
        <div class="col-md-12 col-lg-12">
            <p><strong class="">{{ _('Access token') }}</strong><br><code>{{token.access_token}}</code></p>
            <div class="alert alert-danger">
                <p>{{ _('Do not share this personal access token. It gives full access to your account.') }}</p>
            </div>
        </div>
    </div>
    {%- block token_footer_text %}{%- endblock %}
</div>
<hr>
{% endif %}
<div class="panel-body">
    <form method="POST" role="form" action="">
    {%- for field in form %}
    {{ render_field(field) }}
    {%- endfor %}
    <div class="form-group">
        <button type="submit" name="delete" class="btn btn-danger"><i class="fa fa-trash-o"></i> {{ _('Delete') }}</button>
        <button type="submit" name="save" class="btn btn-primary"><i class="fa fa-check"></i> {{ _('Save') }}</button>
    </div>
    </form>
</div>
{{helpers.panel_end(with_body=False)}}
{% endblock %}
