{# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
#}

{% macro render_field(field, icon="", placeholder='', autofocus=False) %}
{%- if field.widget.input_type == 'hidden' %}
  {{ field() }}
{%- else %}
  <div class="form-group {% if icon %} has-feedback{% endif %}{% if field.errors %} has-error{% endif %}">
    {{ field.label }}
    {%- set extras = dict(autofocus="") if autofocus else dict() %}
    {{field(class_="form-control", placeholder=placeholder, **extras)}}
    {%- if icon %}
    <i class="{{icon}} form-control-feedback" aria-hidden="true" ></i>
    {%- endif %}
    {%- if field.description %}
    <span class="help-block"><small>{{ field.description }}</small></span>
    {%- endif %}
    {%- if field.errors %}
    <div class="alert alert-danger alert-dismissible text-left" role="alert">
    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
    {%- for error in field.errors %}
    <p>{{error}}</p>
    {% endfor %}
    </div>
    {%- endif %}
  </div>
{%- endif %}
{% endmacro %}
