{#
# This file is part of Invenio.
# Copyright (C) 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#}

{%- extends config.OAUTH2SERVER_BASE_TEMPLATE %}

{%- import "invenio_oauth2server/settings/helpers.html" as helpers with context %}

{%- block page_body %}
<div class="row">
  <div class="col-md-8 col-md-offset-2">
  {{ helpers.panel_start(
    _('Invalid authorization request'),
    icon='fa fa-warning fa-fw'
  ) }}

  {{ _('Invalid authorization request') }}

  {{ _('The service that redirected your here made an invalid authorization request (error code: %(x_error)s).',
       x_error=error.error) }}

  <div class="row">
    <hr />
    <div align="center">
      <a href="/" class="btn btn-info">
        {{ _('Get me out of here!') }}
      </a>
    </div>
    {{ helpers.panel_end() }}
  </div>
</div>
{%- endblock %}
