{#
# This file is part of Invenio.
# Copyright (C) 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#}


{%- extends "invenio_oauth2server/settings/index.html" %}

{%- import "invenio_oauth2server/settings/helpers.html" as helpers with context %}
{%- from "invenio_oauth2server/settings/_macros.html" import render_field with context %}

{% block settings_content %}
{{ helpers.panel_start(
    _('New personal access token'),
) }}
<form role="form" method="POST">
{%- for field in form %}
{{ render_field(field) }}
{%- endfor %}
<div class="form-group">
    <div class="">
      <a href="{{url_for('.index')}}" class="btn btn-default"><i class="fa fa-times"></i> {{ _('Cancel') }}</a>
      <button type="submit" class="btn btn-primary"><i class="fa fa-check"></i> {{ _('Create') }}</button>
    </div>
  </div>
</form>
{{helpers.panel_end()}}
{% endblock %}
